@@ -20,7 +20,7 @@
 #define WEBKIT_DOM_USE_UNSTABLE_API
 #include <webkitdom/WebKitDOMElementUnstable.h>
 #include <webkitdom/WebKitDOMDOMWindowUnstable.h>
-#include <JavaScriptCore/JavaScript.h>
+#include <jsc/jsc.h>
 
 #include "extension/clib/dom_element.h"
 #include "extension/clib/dom_document.h"
@@ -31,7 +31,7 @@
 
 static lua_class_t dom_element_class;
 
-LUA_OBJECT_FUNCS(dom_element_class, dom_element_t, dom_element);
+LUA_DOM_ELEMENT_FUNCS(dom_element_class, dom_element_t, dom_element);
 
 static dom_element_t*
 luaH_check_dom_element(lua_State *L, gint udx)
@@ -42,6 +42,42 @@ luaH_check_dom_element(lua_State *L, gint udx)
     return element;
 }
 
+static gboolean
+dom_element_collect_event_keys(gpointer key, gpointer UNUSED(value), GPtrArray *keys)
+{
+    g_ptr_array_add(keys, key);
+    return FALSE;
+}
+
+/* forward declarations of callbacks */
+static void event_listener_capture_cb(WebKitDOMElement *elem, WebKitDOMEvent *event, dom_element_t *element);
+static void event_listener_bubble_cb(WebKitDOMElement *elem, WebKitDOMEvent *event, dom_element_t *element);
+
+static void
+dom_element_unregister_webkit_event_listeners(dom_element_t *element)
+{
+    if (element && element->element && element->dom_events) {
+        WebKitDOMEventTarget *target = WEBKIT_DOM_EVENT_TARGET(element->element);
+        if (target) {
+            guint i;
+            GPtrArray *keys = g_ptr_array_new();
+            /* collect all existing webkit listener's types registerd for this element */
+            g_tree_foreach(element->dom_events, (GTraverseFunc)dom_element_collect_event_keys, keys);
+            /* remove all registered webkit listeners for both capture and bubble phases */
+            for (i = 0; i < keys->len; i++) {
+                char *type = g_ptr_array_index(keys, i);
+                if ( g_str_has_suffix(type, "::capture" ) )
+                    webkit_dom_event_target_remove_event_listener(target, type,
+                                                                  G_CALLBACK(event_listener_capture_cb), TRUE);
+                else
+                    webkit_dom_event_target_remove_event_listener(target, type,
+                                                                  G_CALLBACK(event_listener_bubble_cb), FALSE);
+            }
+            g_ptr_array_free(keys, FALSE);
+        }
+    }
+}
+
 static void
 webkit_web_page_destroy_cb(dom_element_t *element, GObject *node)
 {
@@ -50,10 +86,25 @@ webkit_web_page_destroy_cb(dom_element_t *element, GObject *node)
     luaH_object_emit_signal(L, -1, "destroy", 0, 0);
     lua_pop(L, 1);
 
+    dom_element_unregister_webkit_event_listeners(element);
+
     element->element = NULL;
     luaH_uniq_del_ptr(common.L, REG_KEY, node);
 }
 
+static gint
+luaH_dom_element_gc(lua_State *L)
+{
+    dom_element_t *element = luaH_checkudata(L, 1, &dom_element_class);
+    if (element) {
+        dom_element_unregister_webkit_event_listeners(element);
+
+        if (element->dom_events)
+            signal_destroy(element->dom_events);
+    }
+    return luaH_object_gc(L);
+}
+
 gint
 luaH_dom_element_from_node(lua_State *L, WebKitDOMElement* node)
 {
@@ -116,40 +167,27 @@ dom_element_selector(dom_element_t *element)
     return sel;
 }
 
-JSValueRef
+JSCValue *
 dom_element_js_ref(page_t *page, dom_element_t *element)
 {
     gchar *sel = dom_element_selector(element);
 
-    /* Get JSValueRef to document.getElementById() */
     WebKitFrame *frame = webkit_web_page_get_main_frame(page->page);
     WebKitScriptWorld *world = extension.script_world;
-    JSGlobalContextRef ctx = webkit_frame_get_javascript_context_for_script_world(frame, world);
+    JSCContext *ctx = webkit_frame_get_js_context_for_script_world(frame, world);
 
-    JSObjectRef js_global = JSContextGetGlobalObject(ctx);
-    JSStringRef doc_key = JSStringCreateWithUTF8CString("document");
-    JSStringRef query_key = JSStringCreateWithUTF8CString("querySelector");
-    JSStringRef sel_key = JSStringCreateWithUTF8CString(sel);
-    JSValueRef sel_val = JSValueMakeString(ctx, sel_key);
+    JSCValue *js_global = jsc_context_get_global_object(ctx);
+    JSCValue *js_doc = jsc_value_object_get_property(js_global, "document");
+    JSCValue *ret = jsc_value_object_invoke_method(js_doc, "querySelector", G_TYPE_STRING, sel, G_TYPE_NONE);
 
-    JSValueRef js_doc = JSObjectGetProperty(ctx, js_global, doc_key, NULL);
-    JSValueRef js_get_elem = JSObjectGetProperty(ctx, (JSObjectRef)js_doc, query_key, NULL);
-    JSValueRef ret = JSObjectCallAsFunction(ctx, (JSObjectRef)js_get_elem, (JSObjectRef)js_doc, 1, &sel_val, NULL);
-
-    JSStringRelease(doc_key);
-    JSStringRelease(query_key);
-    JSStringRelease(sel_key);
+    g_object_unref(js_doc);
+    g_object_unref(js_global);
+    g_object_unref(ctx);
     g_free(sel);
 
     return ret;
 }
 
-static gint
-luaH_dom_element_gc(lua_State *L)
-{
-    return luaH_object_gc(L);
-}
-
 static gint
 luaH_dom_element_query(lua_State *L)
 {
@@ -359,13 +397,102 @@ luaH_dom_element_submit(lua_State *L)
     return 0;
 }
 
+/* Emit a dom event to an object.
+ * `event` is the webkit dom event.
+ * `oud` is the object index on the stack.
+ * `name` is the name of the signal.
+ */
+static gint
+luaH_dom_element_emit_dom_event(lua_State *L, WebKitDOMEvent *event, gint oud, const gchar *name) {
+    gint nargs = 1;
+    gint nret = 0;
+    gint ret, top, bot = lua_gettop(L) - nargs + 1;
+    gint oud_abs = luaH_absindex(L, oud);
+    dom_element_t *obj = luaH_check_dom_element(L, oud);
+
+    gchar *origin = luaH_callerinfo(L);
+    debug("emit dom event " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
+            " on %p from "
+            ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET " (%d args, %d nret)",
+            name, obj, origin ? origin : "<GTK>", nargs, nret);
+    g_free(origin);
+
+    if(!obj)
+        return luaL_error(L, "trying to emit dom event " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET " on non-object", name);
+
+    signal_array_t *sigfuncs = signal_lookup(obj->dom_events, name);
+    if (sigfuncs) {
+        guint nbfunc = sigfuncs->len;
+        luaL_checkstack(L, lua_gettop(L) + nbfunc + nargs + 2,
+                "too many signal handlers; need a new implementation!");
+        /* Push all functions and then execute, because this list can change
+         * while executing funcs. */
+        for (guint i = 0; i < nbfunc; i++)
+            luaH_object_push_item(L, oud_abs, sigfuncs->pdata[i]);
+
+        gboolean cancel = false;
+        for (guint i = 0; i < nbfunc; i++) {
+            /* push object */
+            lua_pushvalue(L, oud_abs);
+            /* push event arg */
+            lua_pushvalue(L, - nargs - nbfunc - 1 + i);
+            /* push first function */
+            lua_pushvalue(L, - nargs - nbfunc - 1 + i);
+            /* remove this first function */
+            lua_remove(L, - nargs - nbfunc - 2 + i);
+            top = lua_gettop(L) - 2 - nargs;
+
+            luaH_dofunction(L, nargs + 1, LUA_MULTRET);
+            ret = lua_gettop(L) - top;
+
+            /* ignore all return values */
+            lua_pop(L, ret);
+
+            /* push event arg */
+            lua_pushvalue(L, - nargs - nbfunc + 1 + i);
+
+            /* check if field 'prevent_default' set to true and if it's set then
+               call webkit_dom_event_prevent_default for event */
+            lua_pushliteral(L, "prevent_default");
+            lua_rawget(L, -2);
+
+            if (lua_toboolean(L, -1)) webkit_dom_event_prevent_default(event);
+            lua_pop(L, 1);
+
+            /* check if field 'cancel' and if it set to true then call
+               stopPropagation */
+            lua_pushliteral(L, "cancel");
+            lua_rawget(L, -2);
+
+            if (lua_toboolean(L, -1)) {
+                webkit_dom_event_stop_propagation(event);
+                cancel = true;
+            }
+
+            /* clean stack from cancel and table*/
+            lua_pop(L, 2);
+
+            /* if even should be canceled then cleanup stack */
+            if (cancel) {
+                for (gint i = bot; i < top; i++)
+                    lua_remove(L, bot);
+                break;
+            }
+        }
+    }
+    lua_pop(L, nargs);
+    return 0;
+}
+
 static void
-event_listener_cb(WebKitDOMElement *UNUSED(elem), WebKitDOMEvent *event, gpointer func)
+event_listener_cb(WebKitDOMElement *UNUSED(elem), WebKitDOMEvent *event, gboolean capture, dom_element_t *element)
 {
     lua_State *L = common.L;
 
+    /* pushing dom element object to lua stack */
+    luaH_uniq_get_ptr(L, REG_KEY, element->element);
+
     lua_createtable(L, 0, 1);
-    lua_pushvalue(L, -1); /* pushing copy of cb argument */
     lua_pushliteral(L, "target");
     WebKitDOMEventTarget *target = webkit_dom_event_get_src_element(event);
     luaH_dom_element_from_node(L, WEBKIT_DOM_ELEMENT(target));
@@ -376,6 +503,13 @@ event_listener_cb(WebKitDOMElement *UNUSED(elem), WebKitDOMEvent *event, gpointe
     lua_pushstring(L, type);
     lua_rawset(L, -3);
 
+    gchar *staged_type = g_strjoin( "::", type, ( capture ? "capture" : "bubble" ), NULL);
+
+    lua_pushliteral(L, "phase");
+    gushort phase = webkit_dom_event_get_event_phase(event);
+    lua_pushinteger(L, phase);
+    lua_rawset(L, -3);
+
     if (WEBKIT_DOM_IS_MOUSE_EVENT(event)) {
         lua_pushliteral(L, "button");
         gushort button = webkit_dom_mouse_event_get_button(WEBKIT_DOM_MOUSE_EVENT(event));
@@ -415,22 +549,67 @@ event_listener_cb(WebKitDOMElement *UNUSED(elem), WebKitDOMEvent *event, gpointe
         lua_rawset(L, -3);
     }
 
-    luaH_object_push(L, func);
-    luaH_dofunction(L, 1, 0);
+    luaH_dom_element_emit_dom_event(L, event, -2, staged_type);
+    g_free(staged_type);
 
-    /* after calling callback examine field 'cancel' in argument
-       passed to callback and if it set to true then call
-       stopPropagation */
-    lua_pushliteral(L, "cancel");
-    lua_rawget(L, -2);
-    if (lua_toboolean(L, -1)) webkit_dom_event_stop_propagation(event);
+    /* pop dom element from stack */
     lua_pop(L, 1);
-    /* also check if prevent_default set to true and if it's set then
-       call webkit_dom_event_prevent_default for event */
-    lua_pushliteral(L, "prevent_default");
-    lua_rawget(L, -2);
-    if (lua_toboolean(L, -1)) webkit_dom_event_prevent_default(event);
-    lua_pop(L, 2);
+
+}
+
+static void
+event_listener_capture_cb(WebKitDOMElement *elem, WebKitDOMEvent *event, dom_element_t *element)
+{
+    return event_listener_cb(elem, event, TRUE, element);
+}
+
+static void
+event_listener_bubble_cb(WebKitDOMElement *elem, WebKitDOMEvent *event, dom_element_t *element)
+{
+    return event_listener_cb(elem, event, FALSE, element);
+}
+
+/* Add a dom event to an object.
+ * `oud` is the object index on the stack.
+ * `name` is the name of the signal.
+ * `ud` is the index of function to call when dom event triggered. */
+void
+luaH_dom_element_add_dom_event(lua_State *L, gint oud,
+        const gchar *name, gint ud) {
+    luaH_checkfunction(L, ud);
+    dom_element_t *obj = luaH_check_dom_element(L, oud);
+
+    gchar *origin = luaH_callerinfo(L);
+    debug("add dom event " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
+            " on %p from " ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET,
+            name, obj, origin);
+    g_free(origin);
+
+    signal_add(obj->dom_events, name, luaH_object_ref_item(L, oud, ud));
+}
+
+/* Remove a dom event from an object.
+ * `oud` is the object index on the stack.
+ * `name` is the name of the signal.
+ * `ud` is the index of function that should be removed.
+ */
+void
+luaH_dom_element_remove_dom_event(lua_State *L, gint oud,
+        const gchar *name, gint ud) {
+    luaH_checkfunction(L, ud);
+    dom_element_t *obj = luaH_check_dom_element(L, oud);
+    gpointer ref = (gpointer) lua_topointer(L, ud);
+
+    gchar *origin = luaH_callerinfo(L);
+    debug("remove dom event " ANSI_COLOR_BLUE "\"%s\"" ANSI_COLOR_RESET
+            " on %p from " ANSI_COLOR_GREEN "%s" ANSI_COLOR_RESET,
+            name, obj, origin);
+    g_free(origin);
+
+    signal_remove(obj->dom_events, name, ref);
+
+    luaH_object_unref_item(L, oud, ref);
+    lua_remove(L, ud);
 }
 
 static gint
@@ -440,31 +619,62 @@ luaH_dom_element_add_event_listener(lua_State *L)
     const gchar *type = luaL_checkstring(L, 2);
     gboolean capture = lua_toboolean(L, 3);
     luaH_checkfunction(L, 4);
-    gpointer func = luaH_object_ref(L, 4);
+    gboolean ret = true;
 
     WebKitDOMEventTarget *target = WEBKIT_DOM_EVENT_TARGET(element->element);
 
-    gboolean ret = webkit_dom_event_target_add_event_listener(target, type,
-            G_CALLBACK(event_listener_cb), capture, func);
+    gchar *staged_type = g_strjoin("::", type, (capture ? "capture" : "bubble" ), NULL);
 
+    /* check if we already have any signals of required type in this dom element */
+    signal_array_t *signals = signal_lookup(element->dom_events, staged_type);
+
+    if (!signals || (signals && signals->len == 0)) {
+        if (capture)
+            ret = webkit_dom_event_target_add_event_listener(target, type,
+                                                             G_CALLBACK(event_listener_capture_cb), capture, element);
+        else
+            ret = webkit_dom_event_target_add_event_listener(target, type,
+                                                             G_CALLBACK(event_listener_bubble_cb), capture, element);
+    }
+
+    luaH_dom_element_add_dom_event(L, 1, staged_type, 4);
+    g_free(staged_type);
+
+    lua_pop(L, 3);
     lua_pushboolean(L, ret);
+
     return 1;
 }
 
-/* because we processing all events in one signle event_listener_cb
-   removing even listener will removes all listeners of this type on
-   element, probably we can somehow scan registered listeners and
-   remove only those one that have func as user data */
 static gint
 luaH_dom_element_remove_event_listener(lua_State *L)
 {
     dom_element_t *element = luaH_check_dom_element(L, 1);
     const gchar *type = luaL_checkstring(L, 2);
     gboolean capture = lua_toboolean(L, 3);
+    luaH_checkfunction(L, 4);
+    gboolean ret = true;
 
-    WebKitDOMEventTarget *target = WEBKIT_DOM_EVENT_TARGET(element->element);
-    gboolean ret = webkit_dom_event_target_remove_event_listener(target, type,
-            G_CALLBACK(event_listener_cb), capture);
+    gchar *staged_type = g_strjoin("::", type, (capture ? "capture" : "bubble" ), NULL);
+
+    /* remove func from dom element`s signals */
+    luaH_dom_element_remove_dom_event(L, 1, staged_type, 4);
+
+    /* retrieve remaining signals for dom element */
+    signal_array_t *signals = signal_lookup(element->dom_events, staged_type);
+
+    g_free(staged_type);
+
+    /* if no more lua signal handlers registered -- remove it in webkit as well */
+    if (!signals || (signals && signals->len == 0)) {
+        WebKitDOMEventTarget *target = WEBKIT_DOM_EVENT_TARGET(element->element);
+        if (capture)
+            ret = webkit_dom_event_target_remove_event_listener(target, type,
+                                                                G_CALLBACK(event_listener_capture_cb), capture);
+        else
+            ret = webkit_dom_event_target_remove_event_listener(target, type,
+                                                                G_CALLBACK(event_listener_bubble_cb), capture);
+    }
 
     lua_pushboolean(L, ret);
     return 1;
@@ -650,7 +860,7 @@ luaH_dom_element_push_document(lua_State *L)
         doc = webkit_dom_html_iframe_element_get_content_document(
                 WEBKIT_DOM_HTML_IFRAME_ELEMENT(element->element));
     } else
-        return 0;
+        doc = webkit_dom_node_get_owner_document(WEBKIT_DOM_NODE(element->element));
 
     return luaH_dom_document_from_webkit_dom_document(L, doc);
 }
