@@ -12,6 +12,7 @@
 local _M = {}
 
 local window = require("window")
+local webview = require("webview")
 local taborder = require("taborder")
 local settings = require("settings")
 
@@ -157,12 +158,12 @@ modes.add_binds("normal", {
 
         #### Example binding
 
-        lousy.bind.key({}, "%", function (w, state)
-            w:scroll{ ypct = state.count }
-        end, { count = 0 })
+            lousy.bind.key({}, "%", function (w, state)
+                w:scroll{ ypct = state.count }
+            end, { count = 0 })
 
-    This binding demonstrates several concepts. Firstly that you are able to
-    specify per-binding default values of `count`. In this case if the user
+        This binding demonstrates several concepts. Firstly that you are able to
+        specify per-binding default values of `count`. In this case if the user
         types `"%"` the document will be scrolled vertically to `0%` (the top).
 
         If the user types `"100%"` then the document will be scrolled to `100%`
@@ -404,7 +405,6 @@ modes.add_binds("normal", {
     -- Window
     { "^ZZ$", "Quit and save the session.", function (w) w:save_session() w:close_win() end },
     { "^ZQ$", "Quit and don't save the session.", function (w) w:close_win() end },
-    { "^D$",  "Quit and don't save the session.", function (w) w:close_win() end },
 
     -- Enter passthrough mode
     { "<Control-z>", "Enter `passthrough` mode, ignores all luakit keybindings.",
@@ -496,7 +496,7 @@ modes.add_cmds({
                     callback = function (_, err)
                         w:error(err)
                     end,
-                }) 
+                })
             else
                 w:error("No argument provided")
             end
@@ -504,11 +504,13 @@ modes.add_cmds({
 
     -- Tab manipulation commands
     { ":tab", "Execute command and open result in new tab.", {
-        func = function (w, o) w:new_tab() w:run_cmd(":" .. o.arg) end,
+        func = function (w, o) if o.arg then w:new_tab() w:run_cmd(":" .. o.arg)
+                                else w:error("Parameter missing") end end,
         format = "{command}",
     }},
     { ":tabd[o]", "Execute command in each tab.", {
-        func = function (w, o) w:each_tab(function () w:run_cmd(":" .. o.arg) end) end,
+        func = function (w, o) if o.arg then w:each_tab(function () w:run_cmd(":" .. o.arg) end)
+                                else w:error("Parameter missing") end end,
         format = "{command}",
     }},
     { ":tabdu[plicate]", "Duplicate current tab.",
@@ -530,27 +532,27 @@ modes.add_cmds({
     end },
 
     { ":lua", "Evaluate Lua snippet.", function (w, o)
-            local a = o.arg
-            if a then
-                -- Parse as expression first, then statement
-                -- With this order an error message won't contain the print() wrapper
-                local ret, err = loadstring("print(" .. a .. ")", "lua-cmd")
-                if err then
-                    ret, err = loadstring(a, "lua-cmd")
-                end
-                if err then
-                    w:error(err)
-                else
-                    setfenv(ret, setmetatable({}, { __index = function (_, k)
-                        if _G[k] ~= nil then return _G[k] end
-                        if k == "w" then return w end
-                        if package.loaded[k] then return package.loaded[k] end
-            end, __newindex = _G }))
-        ret()
-    end
-else
-    w:set_mode("lua")
-end
+        local a = o.arg
+        if a then
+            -- Parse as expression first, then statement
+            -- With this order an error message won't contain the print() wrapper
+            local ret, err = loadstring("print(" .. a .. ")", "lua-cmd")
+            if err then
+                ret, err = loadstring(a, "lua-cmd")
+            end
+            if err then
+                w:error(err)
+            else
+                setfenv(ret, setmetatable({}, { __index = function (_, k)
+                    if _G[k] ~= nil then return _G[k] end
+                    if k == "w" then return w end
+                    if package.loaded[k] then return package.loaded[k] end
+                end, __newindex = _G }))
+                ret()
+            end
+        else
+            w:set_mode("lua")
+        end
     end },
 
     { ":dump", "Dump current tabs html to file.",
@@ -569,6 +571,25 @@ end
                 luakit.idle_add(function () coroutine.resume(co) end)
             end
         end },
+
+    { ":save", "Save page as shown to file.",
+        function (w, o)
+            local fname = string.gsub(w.win.title, '[^%w%.%-]', '_')..'.mhtml' -- sanitize filename
+            local file = o.arg or luakit.save_file("Save file", w.win, xdg.download_dir or '.', fname)
+            if file then
+                local view = w.view
+                -- FIXME: note that this is called after all calls
+                -- to luakit.save_file(), including those not called
+                -- by :save; the better way to do this is to make
+                -- save_file() return an ID, store that in a table, and
+                -- check that table before showing a notification.
+                view:add_signal("save-finished", function(v, f, err)
+                    local ww = webview.window(v)
+                    ww:notify(err or ("Saved to: " .. f))
+                end)
+                view:save(file)
+            end
+        end },
 })
 
 local function convert (str, new_type)
