@@ -145,6 +145,12 @@ local init_funcs = {
             end
         end)
     end,
+
+    enable_pdfjs = function (view)
+        local pdfjs_enabled =
+            settings.get_setting("application.enable_pdfjs")
+        view:set_pdfjs(pdfjs_enabled or false)
+    end,
 }
 
 --- These methods are present when you index a window instance and no window
@@ -552,11 +558,6 @@ local webview_settings = {
         default = false,
         desc = "Whether to allow web pages to access audio and video devices for capture.",
     },
-    ["webview.enable_offline_web_application_cache"] = {
-        type = "boolean",
-        default = true,
-        desc = "Whether to enable offline web application support." ,
-    },
     ["webview.enable_page_cache"] = {
         type = "boolean",
         default = true,
@@ -567,6 +568,14 @@ local webview_settings = {
             Disabling this setting is only useful to conserve memory.
         ]=],
     },
+    ["application.enable_pdfjs"] = {
+        type = "boolean",
+        default = true,
+        desc = [=[
+            Whether to load PDFs in a built-in javascript renderer.
+            The default is to pass them on to the OS.  This setting
+            only becomes active in new tabs.]=]
+    },
     ["webview.enable_plugins"] = {
         type = "boolean",
         default = true,
@@ -633,7 +642,7 @@ local webview_settings = {
             ["always"] = { desc = "Always enable hardware acceleration.", label = "Always", },
             ["never"] = { desc = "Always disable hardware acceleration.", label = "Never", },
         },
-        default = "on-demand",
+        default = "always",
         desc = "The policy used to determine when hardware acceleration should be used to render web content.",
     },
     ["webview.javascript_can_access_clipboard"] = {
@@ -646,11 +655,6 @@ local webview_settings = {
         default = false,
         desc = "Whether JavaScript can open windows without user intervention.",
     },
-    ["webview.load_icons_ignoring_image_load_setting"] = {
-        type = "boolean",
-        default = false,
-        desc = "Whether web page favicons should be loaded, even if `webview.auto_load_images` is disabled.",
-    },
     ["webview.media_playback_allows_inline"] = {
         type = "boolean",
         default = true,
@@ -696,11 +700,11 @@ local webview_settings = {
         default = 100,
         desc = "The default zoom level, as a percentage, at which to draw content.",
     },
-    ["webview.zoom_text_only"] = {
-        type = "boolean",
-        default = false,
-        desc = "Whether zooming the page should affect the size of all elements, or only the text content.",
-    },
+--    ["webview.zoom_text_only"] = {
+--        type = "boolean",
+--        default = false,
+--        desc = "Whether zooming the page should affect the size of all elements, or only the text content.",
+--    },
 }
 settings.register_settings(webview_settings)
 settings.register_settings({
@@ -717,10 +721,15 @@ settings.register_settings({
 
 _M.add_signal("init", function (view)
     local set = function (wv, k, v, match)
-        if v ~= nil then
-            k = k:sub(9) -- Strip off "webview." prefix
+        -- hand through webview.-prefixed settings
+        if v ~= nil and k:find('webview.', 1, true) then
+            k = k:sub(9) -- Strip off prefix
             if k == "zoom_level" then v = v/100.0 end
             if k == "user_agent" and v == "" then v = nil end
+            -- bug: when zoom_text_only is loaded after zoom_level,
+            -- the zoom_level resets to 0.5. Keep zoom_text_only ignored
+            -- until the settings can be applied in a controlled order
+            if k == "zoom_text_only" then return end
             match = match and (" (matched '"..match.."')") or ""
             msg.verbose("setting property %s = %s" .. match, k, v, match)
             wv[k] = v
