@@ -245,6 +245,11 @@ local init_funcs = {
         }) do wi.font = v end
     end,
 
+    set_dark_mode = function (w)
+        local dark_mode = settings.get_setting("application.prefer_dark_mode")
+            w.win:set_dark_mode(dark_mode)
+    end,
+
     set_default_size = function (w)
         local size = settings.get_setting("window.new_window_size")
         if string.match(size, "^%d+x%d+$") then
@@ -463,6 +468,9 @@ _M.methods = {
         if not view then
             -- Make new webview widget
             view = webview.new({ private = opts.private })
+            if not arg and not opts.no_initial_url then
+                view.uri = settings.get_setting("window.new_tab_page")
+            end
             w:attach_tab(view, switch, order)
         end
 
@@ -472,6 +480,7 @@ _M.methods = {
             w:search_open_navigate(view, arg)
         end
 
+        view:reload()
         return view
     end,
 
@@ -777,6 +786,11 @@ function _M.ancestor(w)
 end
 
 settings.register_settings({
+    ["application.prefer_dark_mode"] = {
+        type = "boolean",
+        default = false,
+        desc = "Perfer dark CSS when the website supports it (requires restart).",
+    },
     ["window.act_on_synthetic_keys"] = {
         type = "boolean",
         default = false,
