@@ -24,7 +24,7 @@
 
 #include "common/luah.h"
 
-void luaH_init();
+void luaH_init(gchar **);
 gboolean luaH_parserc(const gchar *, gboolean);
 gint luaH_mtnext(lua_State *, gint);
 
