@@ -118,7 +118,6 @@ static property_t webview_settings_properties[] = {
   { L_TK_ENABLE_JAVASCRIPT,                         "enable-javascript",                         BOOL,  TRUE },
   { L_TK_ENABLE_MEDIA_STREAM,                       "enable-media-stream",                       BOOL,  TRUE },
   { L_TK_ENABLE_MEDIASOURCE,                        "enable-mediasource",                        BOOL,  TRUE },
-  { L_TK_ENABLE_OFFLINE_WEB_APPLICATION_CACHE,      "enable-offline-web-application-cache",      BOOL,  TRUE },
   { L_TK_ENABLE_PAGE_CACHE,                         "enable-page-cache",                         BOOL,  TRUE },
   { L_TK_ENABLE_PLUGINS,                            "enable-plugins",                            BOOL,  TRUE },
   /* replaces resizable-text-areas */
@@ -134,7 +133,6 @@ static property_t webview_settings_properties[] = {
   { L_TK_FANTASY_FONT_FAMILY,                       "fantasy-font-family",                       CHAR,  TRUE },
   { L_TK_JAVASCRIPT_CAN_ACCESS_CLIPBOARD,           "javascript-can-access-clipboard",           BOOL,  TRUE },
   { L_TK_JAVASCRIPT_CAN_OPEN_WINDOWS_AUTOMATICALLY, "javascript-can-open-windows-automatically", BOOL,  TRUE },
-  { L_TK_LOAD_ICONS_IGNORING_IMAGE_LOAD_SETTING,    "load-icons-ignoring-image-load-setting",    BOOL,  TRUE },
   { L_TK_MEDIA_PLAYBACK_ALLOWS_INLINE,              "media-playback-allows-inline",              BOOL,  TRUE },
   { L_TK_MEDIA_PLAYBACK_REQUIRES_GESTURE,           "media-playback-requires-user-gesture",      BOOL,  TRUE },
   { L_TK_MINIMUM_FONT_SIZE,                         "minimum-font-size",                         INT,   TRUE },
@@ -758,6 +756,29 @@ permission_request_cb(WebKitWebView *UNUSED(v), WebKitPermissionRequest *request
     return ret > 0;
 }
 
+static gint
+luaH_webview_set_pdfjs(lua_State *L)
+{
+    webview_data_t *d = luaH_checkwvdata(L, 1);
+    gboolean enabled = luaH_checkboolean(L, 2);
+
+    g_autoptr(WebKitFeatureList) features
+        = webkit_settings_get_all_features();
+    WebKitSettings *settings = webkit_web_view_get_settings(d->view);
+
+    for (gsize i = 0; i < webkit_feature_list_get_length(features); i++) {
+        WebKitFeature *feature = webkit_feature_list_get(features, i);
+        if (!strcmp(
+                webkit_feature_get_identifier(feature),
+                "PdfJSViewer")) {
+            webkit_settings_set_feature_enabled(settings, feature, enabled);
+            break;
+        }
+    }
+
+    return 0;
+}
+
 static gint
 luaH_webview_index(lua_State *L, widget_t *w, luakit_token_t token)
 {
@@ -798,6 +819,7 @@ luaH_webview_index(lua_State *L, widget_t *w, luakit_token_t token)
       PF_CASE(CLOSE_INSPECTOR,      luaH_webview_close_inspector)
 
       PF_CASE(ALLOW_CERTIFICATE,    luaH_webview_allow_certificate)
+      PF_CASE(SET_PDFJS,            luaH_webview_set_pdfjs)
 
       /* push string properties */
       PS_CASE(HOVERED_URI,          d->hover)
